//
// Created by philipp on 22.03.18.
//

#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>

#include "logging/Logging.h"

PLUGINLIB_EXPORT_CLASS(car::Logging, nodelet::Nodelet);

namespace car
{

constexpr std::size_t Logging::PORT;
constexpr std::size_t Logging::MAX_MESSAGE_SIZE;
constexpr std::size_t Logging::MAX_BUFFER_SIZE;

Logging::Logging(ros::NodeHandle & nh, std::string & name)
    : nh_(nh)
      , name_(name)
      , loggingBuffer(MAX_BUFFER_SIZE)
      , loggingStream(&loggingBuffer)
{}

Logging::Logging()
    : loggingBuffer(MAX_BUFFER_SIZE)
      , loggingStream(&loggingBuffer)
{}

Logging::~Logging()
{}

void Logging::onInit()
{
    NODELET_INFO("Logging::onInit -- START");

    camData = nh_.subscribe("camData", 1, &Logging::camDataCallback, this);
    ccData = nh_.subscribe("ccData", 1, &Logging::ccDataCallback, this);
    environmentData = nh_.subscribe("environmentData", 1, &Logging::environmentDataCallback, this);
    laneData = nh_.subscribe("laneData", 1, &Logging::laneDataCallback, this);
    logEnabled = nh_.subscribe("logEnabled", 1, &Logging::logEnabledCallback, this);
    logString = nh_.subscribe("logString", 1, &Logging::logStringCallback, this);
    rcEnabled = nh_.subscribe("rcEnabled", 1, &Logging::rcEnabledCallback, this);
    stmData = nh_.subscribe("stmData", 1, &Logging::stmDataCallback, this);
    ussData = nh_.subscribe("ussData", 1, &Logging::ussDataCallback, this);

    loggingServer = networking::service::Server<LoggingService>::create(net, PORT, MAX_MESSAGE_SIZE);
    loggingServer->advertiseService(
        [this](const auto & clientEndpoint, auto & requestMessage, auto & responseMessage)
        {
            std::lock_guard<std::mutex> lock{loggingMutex};
            responseMessage = networking::utils::stringFromStreambuf(loggingBuffer,
                                                                     std::min(loggingBuffer.size(), MAX_MESSAGE_SIZE));
        });

    NODELET_INFO("Logging::onInit -- END");
}

void Logging::logMessage(const std::string & msg)
{
    std::lock_guard<std::mutex> lock{loggingMutex};
    loggingStream << msg;
}

void Logging::ccDataCallback(const ccDataMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new cc data ( ).\n";
}

void Logging::camDataCallback(const camDataMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new cam data ( ).\n";
}

void Logging::environmentDataCallback(const environmentDataMsg::ConstPtr & inMsg)
{
    std::cout << "Logging received new environment data ( ).\n";
    logMessage(std::string{"Environment [distance]: "} + std::to_string(inMsg->distance));
}

void Logging::laneDataCallback(const laneDataMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new lane data ( ).\n";
}

void Logging::logEnabledCallback(const logEnabledMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new log enabled ( ).\n";
}

void Logging::logStringCallback(const logStringMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new log string ( ).\n";
}

void Logging::rcEnabledCallback(const rcEnabledMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new rc enabled ( ).\n";
}

void Logging::stmDataCallback(const stmDataMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new stm data ( ).\n";
}

void Logging::ussDataCallback(const ussDataMsg::ConstPtr & inMsg)
{
    // std::cout << "Logging received new uss data ( ).\n";
}

}
